#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.

from typing import List, Optional

import weshort
from weshort.exception import WeShortError
from weshort.types import Payment, Response


class GetPayment:
    async def getPayments(self: "weshort.WeShort") -> List[Payment]:
        """Get All Payments, the main means for interacting with API WeShort.

        Returns:
            ``List[Payment]``: List of Payment.

        Example:
            >>> from weshort import WeShort
            >>> 
            >>> weShort = WeShort(apiToken="YOUR_API_TOKEN")
            >>> try:
            >>>     url = await weShort.getPayments()
            >>>     print(url) # output: List[Payment]
            >>> except WeShortError as e:
            >>>     print(e)
        """
        res = await self.get(
            f"/payment",
        )
        if not isinstance(res, Response):
            raise WeShortError(res)
        shortsUrl: List[Payment] = [Payment(**x) for x in res.responseData]
        return shortsUrl

    async def getPayment(self: "weshort.WeShort", orderId: str) -> Payment:
        """Get Detail Payment, the main means for interacting with API WeShort.

        Parameters:
            orderId (``str``):
                Order ID for Get Detail Payment.
                e.g.: "xxxxxxxxxx1e2e0c3a062b17025exxxxxxxxxx764b505b5c7053405e351c1a1d5c740e".

        Returns:
            ``Payment``: Payment class.

        Example:
            >>> from weshort import WeShort
            >>> 
            >>> weShort = WeShort(apiToken="YOUR_API_TOKEN")
            >>> try:
            >>>     url = await weShort.getPayment("xxxxxxxxxx1e2e0c3a062b17025exxxxxxxxxx764b505b5c7053405e351c1a1d5c740e")
            >>>     print(url) # output: Payment class
            >>> except WeShortError as e:
                    print(e)
        """
        res = await self.get(
            f"/payment?orderId={orderId}",
        )
        if not isinstance(res, Response):
            raise WeShortError(res)
        return Payment(**res.responseData)



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.


class Response:
    def __init__(self, **kwargs):
        self.responseCode: int = kwargs.get("responseCode", 0)
        self.responseSuccess: bool = kwargs.get("responseSuccess", False)
        self.responseMessage: str = kwargs.get("responseMessage", "")
        self.responseData: dict = kwargs.get("responseData", None)

    def parser(self):
        return {
            "_": "Response",
            "responseCode": self.responseCode,
            "responseSuccess": self.responseSuccess,
            "responseMessage": self.responseMessage,
            "responseData": self.responseData
        }



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.


class GetMe:
    def __init__(self, **kwargs):
        self.id: int = kwargs.get("id", 0)
        self.name: str = kwargs.get("name", "")
        self.email: str = kwargs.get("email", "")
        self.avatar: str = kwargs.get("avatar", "")
        self.saldo: int = kwargs.get("saldo", 0)
        self.earnings: int = kwargs.get("penghasilan", 0)
        self.role: str = kwargs.get("role", "")
        self.createdAt: str = kwargs.get("createdAt", "")
        self.updatedAt: str = kwargs.get("updatedAt", "")

    def parser(self):
        return {
            "_": "GetMe",
            "id": self.id,
            "name": self.name,
            "email": self.email,
            "avatar": self.avatar,
            "saldo": self.saldo,
            "earnings": self.earnings,
            "role": self.role,
            "createdAt": self.createdAt,
            "updatedAt": self.updatedAt
        }



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.

from typing import List

import weshort
from weshort.exception import WeShortError
from weshort.types import CreatedPayment, Response


class CreatePayment:
    async def createPayment(self: "weshort.WeShort", shortUrl: str) -> CreatedPayment:
        """Create Payments Shortlink, the main means for interacting with API WeShort.
        
        Parameters:
            shortUrl (``str``):
                Short URL for Create Payments.
                e.g.: "https://weshort.pro/xxxxxx".

        Returns:
            ``CreatedPayment``: CreatedPayment object.

        Example:
            >>> from weshort import WeShort
            >>> 
            >>> weShort = WeShort(apiToken="YOUR_API_TOKEN")
            >>> try:
            >>>     data = await weShort.createPayment("https://weshort.pro/tyrqopeb")
            >>>     print(data) # output: CreatedPayment object
            >>> except WeShortError as e:
            >>>     print(e)
        """
        res = await self.post(
            f"/payment?shortUrl={shortUrl}",
        )
        if not isinstance(res, Response):
            raise WeShortError(res)
        return CreatedPayment(**res.responseData)



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.

from typing import List

import weshort
from weshort.exception import WeShortError
from weshort.types import ShortUrl, Response


class GetShortUrl:
    async def getShortsUrl(self: "weshort.WeShort") -> List[ShortUrl]:
        """Get All Short URL, the main means for interacting with API WeShort.

        Returns:
            ``List[ShortUrl]``: List of ShortUrl.

        Example:
            >>> from weshort import WeShort
            >>> 
            >>> weShort = WeShort(apiToken="YOUR_API_TOKEN")
            >>> try:
            >>>     url = await weShort.getShortUrl("pZO1WopFBjU2B6XJ")
            >>>     print(url) # output: List[ShortUrl]
            >>> except WeShortError as e:
            >>>     print(e)
        """
        res = await self.get(
            f"/short",
        )
        if not isinstance(res, Response):
            raise WeShortError(res)
        shortsUrl: List[ShortUrl] = [
            ShortUrl(
                **x,
                shortUrl=f"https://weshort.pro/{x['keyword']}"
            ) for x in res.responseData
        ]
        return shortsUrl

    async def getShortUrl(self: "weshort.WeShort", keyword: str) -> ShortUrl:
        """Get Short URL, the main means for interacting with API WeShort.

        Parameters:
            keyword (``str``):
                Keyword for Get Short URL.
                e.g.: "pZO1WopFBjU2B6XJ".

        Returns:
            ``ShortUrl``: ShortUrl class.

        Example:
            >>> from weshort import WeShort
            >>> 
            >>> weShort = WeShort(apiToken="YOUR_API_TOKEN")
            >>> try:
            >>>     url = await weShort.getShortUrl("pZO1WopFBjU2B6XJ")
            >>>     print(url) # output: ShortUrl
            >>> except WeShortError as e:
                    print(e)
        """
        res = await self.get(
            f"/short?keyword={keyword}",
        )
        if not isinstance(res, Response):
            raise WeShortError(res)
        return ShortUrl(**res.responseData)



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.

import aiofiles
import aiohttp
import os
from typing import Optional

from .base import Base
from .exception import WeShortError
from .methods import Methods
from .types import GetMe, Response


class WeShort(Base, Methods):
    """WeShort Client, the main means for interacting with API WeShort.

    Parameters:
        apiToken (``str``):
            API token for Authorization Users, e.g.: "AxD_ABCDEFghIklzyxWvuew".
            Get the ``API Token`` in [WeShortProfile](https://weshort.pro).

    Example:
        >>> from weshort import WeShort
        >>> 
        >>> weShort = WeShort(
        >>>     apiToken="YOUR_API_TOKEN"
        >>> )
    """
    def __init__(
        self,
        apiToken: str
    ):
        super().__init__(apiToken=apiToken)

    async def getMe(self) -> GetMe:
        res = await self.get("/client")
        if not isinstance(res, Response):
            raise WeShortError("Failed to get User Info")
        return GetMe(**res.responseData)

    async def deleteAccount(self):
        res = await self.delete("/client")
        if not isinstance(res, Response):
            raise WeShortError("Failed to delete Account")
        return True

    async def generateQris(self, qrisToken: str, uniqueId: Optional[str] = None) -> str:
        if not os.path.exists("downloads"):
            os.makedirs("downloads")
        if not uniqueId:
            # Create Unique Id from qrisToken
            number = "1 2 3 4 5 6 7 8 9 0".split(" ")
            uniqueId = "".join(qrisToken[int(i*3)] for i in number)
        async with aiohttp.ClientSession() as session:
            res = await session.get(
                f"{self.baseUrl}/qris?token={qrisToken}",
                headers=self.headers
            )
            if res.status == 200:
                async with aiofiles.open(f"downloads/WeShortQris-{uniqueId}.png", "wb") as f:
                    async for chunk, _ in res.content.iter_chunks():
                        await f.write(chunk)

        return f"downloads/WeShort-{uniqueId}-Qris.png"



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.


class Withdraw:
    def __init__(self, **kwargs):
        self.id: int = kwargs.get("id", 0)
        self.clientName: str = kwargs.get("clientName", "")
        self.withdrawId: str = kwargs.get("withdrawId", "")
        self.method: str = kwargs.get("method", "")
        self.noRek: str = kwargs.get("noRek", "")
        self.nameRek: str = kwargs.get("nameRek", "")
        self.amount: int = kwargs.get("amount", 0)
        self.feeAdmin: int = kwargs.get("feeAdmin", 0)
        self.status: str = kwargs.get("status", "")
        self.createdAt: str = kwargs.get("createdAt", "")
        self.updatedAt: str = kwargs.get("updatedAt", "")

    def parser(self):
        return {
            "_": "Withdraw",
            "id": self.id,
            "clientName": self.clientName,
            "withdrawId": self.withdrawId,
            "method": self.method,
            "noRek": self.noRek,
            "nameRek": self.nameRek,
            "amount": self.amount,
            "feeAdmin": self.feeAdmin,
            "status": self.status,
            "createdAt": self.createdAt,
            "updatedAt": self.updatedAt
        }



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.

import aiohttp
from typing import Optional

from .exception import ResponseError
from .types import Response
from .version import __version__


class Base:
    version = __version__
    apiToken: str
    baseUrl: str = "https://api.weshort.pro/api"
    def __init__(self, apiToken: str):
        self.apiToken = apiToken
        self.headers = {
            "Content-Type": "application/json",
            "Xd-Token": self.apiToken
        }

    async def post(self, path: str, data: Optional[dict] = None, headers: Optional[dict] = None) -> Response:
        async with aiohttp.ClientSession(headers=self.headers) as session:
            res = await session.post(
                url=f"{self.baseUrl}{path}",
                json=data,
                headers=headers
            )
            json = await res.json()
            response: Response = Response(**json)
            if response.responseSuccess:
                return response
            else:
                raise ResponseError(response.responseMessage)

    async def get(
        self,
        path: str,
        data: Optional[dict] = None,
        headers: Optional[dict] = None
    ):
        async with aiohttp.ClientSession(headers=self.headers) as session:
            req = await session.get(url=f"{self.baseUrl}{path}", json=data, headers=headers)
            json = await req.json()
            response: Response = Response(**json)
            if response.responseSuccess:
                return response
            else:
                raise ResponseError(response.responseMessage)

    async def delete(
        self,
        path: str,
        data: Optional[dict] = None,
        headers: Optional[dict] = None
    ):
        async with aiohttp.ClientSession(headers=self.headers) as session:
            req = await session.delete(url=f"{self.baseUrl}{path}", json=data, headers=headers)
            json = await req.json()
            response: Response = Response(**json)
            if response.responseSuccess:
                return response
            else:
                raise ResponseError(response.responseMessage)



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.

class CreatedPayment:
    def __init__(self, **kwargs):
        self.ownerName: str = kwargs.get("ownerName", "")
        self.transactionId: str = kwargs.get("transactionId", "")
        self.orderId: str = kwargs.get("orderId", "")
        self.qrisToken: str = kwargs.get("qrisToken", "")
        self.expired: str = kwargs.get("expired", "")

    def parser(self):
        return {
            "_": "CreatePayment",
            "ownerName": self.ownerName,
            "orderId": self.orderId,
            "transactionId": self.transactionId,
            "qrisToken": self.qrisToken,
            "expired": self.expired
        }


class Payment:
    def __init__(self, **kwargs):
        self.id: int = kwargs.get("id", 0)
        self.orderId: str = kwargs.get("orderId", "")
        self.ownerName: str = kwargs.get("ownerName", "")
        self.amount: int = kwargs.get("amount", 0)
        self.keyword: str = kwargs.get("keyword", "")
        self.status: str = kwargs.get("status", "")
        self.createdAt: str = kwargs.get("createdAt", "")
        self.updatedAt: str = kwargs.get("updatedAt", "")

    def parser(self):
        return {
            "_": "Payment",
            "id": self.id,
            "orderId": self.orderId,
            "ownerName": self.ownerName,
            "amount": self.amount,
            "keyword": self.keyword,
            "status": self.status,
            "createdAt": self.createdAt,
            "updatedAt": self.updatedAt
        }


class CheckedTransaction:
    def __init__(self, **kwargs):
        self.statusCode: str = kwargs.get("statusCode", "")
        self.transactionId: str = kwargs.get("transactionId", "")
        self.amount: str = kwargs.get("amount", "")
        self.currency: str = kwargs.get("currency", "")
        self.orderId: str = kwargs.get("orderId", "")
        self.paymentType: str = kwargs.get("paymentType", "")
        self.transactionStatus: str = kwargs.get("transactionStatus", "")
        self.fraudStatus: str = kwargs.get("fraudStatus", "")
        self.transactionTime: str = kwargs.get("transactionTime", "")
        self.expiredTime: str = kwargs.get("expiredTime", "")

    def parser(self):
        return {
            "_": "CheckedTransaction",
            "statusCode": self.statusCode,
            "transactionId": self.transactionId,
            "amount": self.amount,
            "currency": self.currency,
            "orderId": self.orderId,
            "paymentType": self.paymentType,
            "transactionStatus": self.transactionStatus,
            "fraudStatus": self.fraudStatus,
            "transactionTime": self.transactionTime,
            "expiredTime": self.expiredTime
        }


class PaymentData:
    def __init__(self, **kwargs):
        self.statusCode: str = kwargs.get("statusCode", "")
        self.transactionId: str = kwargs.get("transactionId", "")
        self.amount: str = kwargs.get("amount", "")
        self.currency: str = kwargs.get("currency", "")
        self.orderId: str = kwargs.get("orderId", "")
        self.paymentType: str = kwargs.get("paymentType", "")
        self.transactionStatus: str = kwargs.get("transactionStatus", "")
        self.fraudStatus: str = kwargs.get("fraudStatus", "")
        self.transactionTime: str = kwargs.get("transactionTime", "")
        self.expiredTime: str = kwargs.get("expiredTime", "")

    def parser(self):
        return {
            "_": "PaymentData",
            "statusCode": self.statusCode,
            "transactionId": self.transactionId,
            "amount": self.amount,
            "currency": self.currency,
            "orderId": self.orderId,
            "paymentType": self.paymentType,
            "transactionStatus": self.transactionStatus,
            "fraudStatus": self.fraudStatus,
            "transactionTime": self.transactionTime,
            "expiredTime": self.expiredTime
        }


class TransactionData:
    def __init__(self, **kwargs):
        self.keyword: str = kwargs.get("keyword", "")
        self.url: str = kwargs.get("url", "")
        self.shortUrl: str = kwargs.get("shortUrl", "")

    def parser(self):
        return {
            "_": "TransactionData",
            "keyword": self.keyword,
            "url": self.url,
            "shortUrl": self.shortUrl
        }


class TransactionSuccess:
    def __init__(self, **kwargs):
        self.payment: PaymentData = PaymentData(**kwargs.get("payment", {}))
        self.data: TransactionData = TransactionData(**kwargs.get("data", {}))

    def parser(self):
        return {
            "_": "TransactionSuccess",
            "payment": self.payment.parser(),
            "data": self.data.parser()
        }



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.

from typing import List

import weshort
from weshort.exception import WeShortError
from weshort.types import Withdraw, Response


class GetWithdraw:
    async def getWithdraws(self: "weshort.WeShort") -> List[Withdraw]:
        """Get All Withdraw, the main means for interacting with API WeShort.

        Returns:
            ``List[Withdraw]``: List of Withdraw

        Example:
            >>> from weshort import WeShort
            >>> 
            >>> weShort = WeShort(apiToken="YOUR_API_TOKEN")
            >>> try:
            >>>     withdraws = await weShort.getWithdraws()
            >>>     print(withdraws) # Output: List[Withdraw]
            >>> except WeShortError as e:
            >>>     print(e)
        """
        res = await self.get(
            f"/wd",
        )
        if not isinstance(res, Response):
            raise WeShortError(res)
        getWithdraws: List[Withdraw] = [Withdraw(**x) for x in res.responseData]
        return getWithdraws

    async def getWithdraw(self: "weshort.WeShort", withdrawId: str) -> Withdraw:
        """Get Withdraw, the main means for interacting with API WeShort.
        
        Parameters:
            withdrawId (``str``):
                Withdraw ID for Getting Details Withdraw.
                e.g.: "WeShortWd_xxxxx_xxxxxx".

        Returns:
            ``Withdraw``: Withdraw Object

        Example:
            >>> from weshort import WeShort
            >>> 
            >>> weShort = WeShort(apiToken="YOUR_API_TOKEN")
            >>> try:
            >>>     withdraw = await weShort.getWithdraw("withdrawId")
            >>>     print(withdraw) # Output: Withdraw
            >>> except WeShortError as e:
                    print(e)
        """
        res = await self.get(
            f"/wd?withdrawId={withdrawId}",
        )
        if not isinstance(res, Response):
            raise WeShortError(res)
        return Withdraw(**res.responseData)



#  Weshort - Shortener URL API Client Library for Python
#  Copyright (C) 2024-present AyiinXd <https://github.com/AyiinXd>
#
#  This file is part of Weshort.
#
#  Weshort is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weshort is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Weshort.  If not, see <http://www.gnu.org/licenses/>.

from typing import Literal

import weshort
from weshort.exception import WeShortError
from weshort.types import Withdraw, Response


class RequestWithdraw:
    async def requestWithdraws(
        self: "weshort.WeShort",
        nameRek: str,
        noRek: str,
        amount: str,
        method: Literal[
            "Dana",
            "Bank BCA",
            "Bank BRI",
            "Bank BNI",
            "Bank Mandiri",
            "Bank SeaBank",
            "Bank Syariah"
        ]
    ) -> Withdraw:
        """Request Withdraw, the main means for interacting with API WeShort.
        
        Parameters:
            nameRek (``str``):
                Name of Recipient.
                e.g.: "AyiinDevs".

            noRek (``str``):
                Number of Recipient.
                e.g.: "123456789".

            amount (``str``):
                Amount of Withdraw.
                e.g.: "1000".

            method (``str``):
                Method of Withdraw.
                e.g.: "Bank SeaBank".

        Returns:
            ``Withdraw``: Withdraw Object

        Example:
            >>> from weshort import WeShort
            >>> 
            >>> weShort = WeShort(apiToken="YOUR_API_TOKEN")
            >>> try:
            >>>     requested = await weShort.requestWithdraws(
            >>>         nameRek="AyiinDevs",
            >>>         noRek="123456789",
            >>>         amount="1000",
            >>>         method="Bank SeaBank"
            >>>     )
            >>>     print(requested) # Output: Withdraw
            >>> except WeShortError as e:
            >>>     print(e)
        """
        res = await self.post(
            f"/wd",
            {
                "nama": nameRek,
                "noRek": noRek,
                "amount": amount,
                "method": method
            }
        )
        if not isinstance(res, Response):
            raise WeShortError(res)
        return Withdraw(**res.responseData)
